@@ -53,6 +53,16 @@
 #undef G_LOG_DOMAIN
 #define G_LOG_DOMAIN "gnc.app-utils.sx"
 
+/** Report errors bilingual:
+ *  in g_critical untranslated and
+ *  in g_list_append translated.
+ */
+#define REPORT_ERROR(list, format, ...) do { \
+    g_critical(format, __VA_ARGS__); \
+    if (list != NULL) \
+        *list = g_list_append(*list, g_strdup_printf(_(format), __VA_ARGS__)); \
+} while (0)
+
 static GObjectClass *parent_class = NULL;
 
 static void gnc_sx_instance_model_class_init (GncSxInstanceModelClass *klass);
@@ -69,6 +79,58 @@ static void _gnc_sx_instance_event_handler(QofInstance *ent, QofEventId event_ty
 
 /* ------------------------------------------------------------ */
 
+static gboolean
+scrub_sx_split_numeric (kvp_frame *kvp, const char *debcred)
+{
+    const gboolean is_credit = g_strcmp0 (debcred, "credit") == 0;
+    const char *formula = is_credit ?
+        GNC_SX_CREDIT_FORMULA : GNC_SX_DEBIT_FORMULA;
+    const char *numeric = is_credit ?
+        GNC_SX_CREDIT_NUMERIC : GNC_SX_DEBIT_NUMERIC;
+    const KvpValue *val = kvp_frame_get_slot_path (kvp,
+                                                   GNC_SX_ID, formula,
+                                                   NULL);
+    const KvpValue *num = kvp_frame_get_slot_path (kvp,
+                                                   GNC_SX_ID, numeric,
+                                                   NULL);
+    const char *value = kvp_value_get_string (val);
+    GHashTable *parser_vars = g_hash_table_new (g_str_hash, g_str_equal);
+    char *error_loc;
+    gnc_numeric amount = gnc_numeric_zero ();
+    const gboolean parse_result =
+        gnc_exp_parser_parse_separate_vars (value, &amount,
+                                            &error_loc, parser_vars);
+    if (!parse_result || g_hash_table_size (parser_vars) != 0)
+        amount = gnc_numeric_zero ();
+    g_hash_table_unref (parser_vars);
+    if (gnc_numeric_eq (amount, kvp_value_get_numeric (num)))
+        return FALSE;
+    kvp_frame_set_slot_path (kvp, kvp_value_new_numeric (amount),
+                             GNC_SX_ID,
+                             numeric,
+                             NULL);
+    return TRUE;
+}
+
+/* Fixes error in pre-2.6.16 where the numeric slot wouldn't get changed if the
+ * formula slot was edited.
+ */
+void
+gnc_sx_scrub_split_numerics (gpointer psplit, gpointer puser)
+{
+    Split *split = GNC_SPLIT (psplit);
+    kvp_frame *kvp = xaccSplitGetSlots (split);
+    Transaction *trans = xaccSplitGetParent (split);
+    gboolean changed;
+    xaccTransBeginEdit (trans);
+    changed = scrub_sx_split_numeric (kvp, "credit") +
+        scrub_sx_split_numeric (kvp, "debit");
+    if (!changed)
+        xaccTransRollbackEdit (trans);
+    else
+        xaccTransCommitEdit (trans);
+}
+
 static void
 _sx_var_to_raw_numeric(gchar *name, GncSxVariable *var, GHashTable *parser_var_hash)
 {
@@ -249,18 +311,16 @@ _get_vars_helper(Transaction *txn, void *var_hash_data)
             ! gnc_commodity_equal(split_cmdty, first_cmdty))
         {
             GncSxVariable *var;
-            GString *var_name;
+            gchar *var_name;
             const gchar *split_mnemonic, *first_mnemonic;
 
-            var_name = g_string_sized_new(16);
             split_mnemonic = gnc_commodity_get_mnemonic(split_cmdty);
             first_mnemonic = gnc_commodity_get_mnemonic(first_cmdty);
-            g_string_printf(var_name, "%s -> %s",
+            var_name = g_strdup_printf ("%s -> %s",
                             split_mnemonic ? split_mnemonic : "(null)",
                             first_mnemonic ? first_mnemonic : "(null)");
-            var = gnc_sx_variable_new(g_strdup(var_name->str));
+            var = gnc_sx_variable_new(var_name);
             g_hash_table_insert(var_hash, g_strdup(var->name), var);
-            g_string_free(var_name, TRUE);
         }
 
     }
@@ -941,14 +1001,10 @@ _get_template_split_account(const SchedXaction* sx, const Split *template_split,
                                       NULL);
     if (kvp_val == NULL)
     {
-        GString *err = g_string_new("");
-        g_string_printf(err, "Null account kvp value for SX [%s], cancelling creation.",
-                        xaccSchedXactionGetName(sx));
-        g_critical("%s", err->str);
-        if (creation_errors != NULL)
-            *creation_errors = g_list_append(*creation_errors, err);
-        else
-            g_string_free(err, TRUE);
+/* Translators: A list of error messages from the Scheduled Transactions (SX).
+ * They might appear in their editor or in "Since last run".                  */
+        gchar *err = N_("Null account kvp value for SX [%s], cancelling creation.");
+        REPORT_ERROR(creation_errors, err, xaccSchedXactionGetName(sx));
         return FALSE;
     }
     acct_guid = kvp_value_get_guid( kvp_val );
@@ -956,16 +1012,9 @@ _get_template_split_account(const SchedXaction* sx, const Split *template_split,
     if (*split_acct == NULL)
     {
         char guid_str[GUID_ENCODING_LENGTH+1];
-        GString *err;
+        gchar* err = N_("Unknown account for guid [%s], cancelling SX [%s] creation.");
         guid_to_string_buff((const GncGUID*)acct_guid, guid_str);
-        err = g_string_new("");
-        g_string_printf(err, "Unknown account for guid [%s], cancelling SX [%s] creation.",
-                        guid_str, xaccSchedXactionGetName(sx));
-        g_critical("%s", err->str);
-        if (creation_errors != NULL)
-            *creation_errors = g_list_append(*creation_errors, err);
-        else
-            g_string_free(err, TRUE);
+        REPORT_ERROR(creation_errors, err, guid_str, xaccSchedXactionGetName(sx));
         return FALSE;
     }
 
@@ -1016,19 +1065,14 @@ _get_sx_formula_value(const SchedXaction* sx, const Split *template_split, gnc_n
                                                 &parseErrorLoc,
                                                 parser_vars))
         {
-            GString *err = g_string_new("");
-            g_string_printf(err, "Error parsing SX [%s] key [%s]=formula [%s] at [%s]: %s",
-                            xaccSchedXactionGetName(sx),
-                            formula_key,
-                            formula_str,
-                            parseErrorLoc,
-                            gnc_exp_parser_error_string());
-            g_critical("%s", err->str);
-            if (creation_errors != NULL)
-                *creation_errors = g_list_append(*creation_errors, err);
-            else
-                g_string_free(err, TRUE);
-        }
+            gchar *err = N_("Error parsing SX [%s] key [%s]=formula [%s] at [%s]: %s.");
+            REPORT_ERROR(creation_errors, err,
+                    xaccSchedXactionGetName(sx),
+                    formula_key,
+                    formula_str,
+                    parseErrorLoc,
+                    gnc_exp_parser_error_string());
+       }
 
         if (parser_vars != NULL)
         {
@@ -1075,15 +1119,9 @@ split_apply_formulas (const Split *split, SxTxnCreationData* creation_data)
     gncn_error = gnc_numeric_check(final);
     if (gncn_error != GNC_ERROR_OK)
     {
-        GString *err = g_string_new("");
-        g_string_printf(err, "error %d in SX [%s] final gnc_numeric value, using 0 instead",
+        gchar *err = N_("Error %d in SX [%s] final gnc_numeric value, using 0 instead.");
+        REPORT_ERROR(creation_data->creation_errors, err,
                         gncn_error, xaccSchedXactionGetName(sx));
-        g_critical("%s", err->str);
-        if (creation_data->creation_errors != NULL)
-            *creation_data->creation_errors =
-                g_list_append(*creation_data->creation_errors, err);
-        else
-            g_string_free(err, TRUE);
         final = gnc_numeric_zero();
     }
     return final;
@@ -1094,12 +1132,12 @@ split_apply_exchange_rate (Split *split, GHashTable *bindings,
                            gnc_commodity *first_cmdty,
                            gnc_commodity *split_cmdty, gnc_numeric *final)
 {
-    GString *exchange_rate_var_name = g_string_sized_new(16);
+    gchar *exchange_rate_var_name;
     GncSxVariable *exchange_rate_var;
     gnc_numeric amt;
     gnc_numeric exchange_rate = gnc_numeric_create (1, 1);
 
-    g_string_printf(exchange_rate_var_name, "%s -> %s",
+    exchange_rate_var_name = g_strdup_printf ("%s -> %s",
                     gnc_commodity_get_mnemonic(first_cmdty),
                     gnc_commodity_get_mnemonic(split_cmdty));
 
@@ -1108,14 +1146,14 @@ split_apply_exchange_rate (Split *split, GHashTable *bindings,
 
     exchange_rate_var =
         (GncSxVariable*)g_hash_table_lookup(bindings,
-                                            exchange_rate_var_name->str);
+                                            exchange_rate_var_name);
 
     if (exchange_rate_var != NULL)
     {
         exchange_rate = exchange_rate_var->value;
         g_debug("exchange_rate is %s", gnc_numeric_to_string (exchange_rate));
     }
-    g_string_free(exchange_rate_var_name, TRUE);
+    g_free (exchange_rate_var_name);
 
     if (!gnc_commodity_is_currency (split_cmdty))
         amt = gnc_numeric_div(*final, exchange_rate,
@@ -1247,7 +1285,7 @@ create_each_transaction_helper(Transaction *template_txn, void *user_data)
 
     if (err_flag)
     {
-        g_critical("new transaction creation sx [%s]",
+        g_critical("Error in SX transaction [%s], creation aborted.",
                    xaccSchedXactionGetName(sx));
         xaccTransDestroy(new_txn);
         xaccTransCommitEdit(new_txn);
@@ -1329,6 +1367,7 @@ gnc_sx_instance_model_effect_change(GncSxInstanceModel *model,
         {
             GncSxInstance *inst = (GncSxInstance*)instance_iter->data;
             gboolean sx_is_auto_create;
+            GList *instance_errors = NULL;
 
             xaccSchedXactionGetAutoCreate(inst->parent->sx, &sx_is_auto_create, NULL);
             if (auto_create_only && !sx_is_auto_create)
@@ -1366,9 +1405,20 @@ gnc_sx_instance_model_effect_change(GncSxInstanceModel *model,
                     increment_sx_state(inst, &last_occur_date, &instance_count, &remain_occur_count);
                     break;
                 case SX_INSTANCE_STATE_TO_CREATE:
-                    create_transactions_for_instance(inst, created_transaction_guids, creation_errors);
-                    increment_sx_state(inst, &last_occur_date, &instance_count, &remain_occur_count);
-                    gnc_sx_instance_model_change_instance_state(model, inst, SX_INSTANCE_STATE_CREATED);
+                    create_transactions_for_instance (inst,
+                                                      created_transaction_guids,
+                                                      &instance_errors);
+                    if (instance_errors == NULL)
+                    {
+                        increment_sx_state (inst, &last_occur_date,
+                                            &instance_count,
+                                            &remain_occur_count);
+                        gnc_sx_instance_model_change_instance_state
+                            (model, inst, SX_INSTANCE_STATE_CREATED);
+                    }
+                    else
+                        *creation_errors = g_list_concat (*creation_errors,
+                                                          instance_errors);
                     break;
                 case SX_INSTANCE_STATE_REMINDER:
                     // do nothing
@@ -1695,30 +1745,20 @@ create_cashflow_helper(Transaction *template_txn, void *user_data)
             gncn_error = gnc_numeric_check(final);
             if (gncn_error != GNC_ERROR_OK)
             {
-                GString *err = g_string_new("");
-                g_string_printf(err, "error %d in SX [%s] final gnc_numeric value, using 0 instead",
-                                gncn_error, xaccSchedXactionGetName(creation_data->sx));
-                g_critical("%s", err->str);
-                if (creation_data->creation_errors != NULL)
-                    *creation_data->creation_errors = g_list_append(*creation_data->creation_errors, err);
-                else
-                    g_string_free(err, TRUE);
+                gchar* err = N_("Error %d in SX [%s] final gnc_numeric value, using 0 instead.");
+                REPORT_ERROR(creation_data->creation_errors, err,
+                             gncn_error, xaccSchedXactionGetName(creation_data->sx));
                 final = gnc_numeric_zero();
             }
 
             /* Print error message if we would have needed an exchange rate */
             if (! gnc_commodity_equal(split_cmdty, first_cmdty))
             {
-                GString *err = g_string_new("");
-                g_string_printf(err, "No exchange rate available in SX [%s] for %s -> %s, value is zero",
-                                xaccSchedXactionGetName(creation_data->sx),
-                                gnc_commodity_get_mnemonic(split_cmdty),
-                                gnc_commodity_get_mnemonic(first_cmdty));
-                g_critical("%s", err->str);
-                if (creation_data->creation_errors != NULL)
-                    *creation_data->creation_errors = g_list_append(*creation_data->creation_errors, err);
-                else
-                    g_string_free(err, TRUE);
+                gchar *err = N_("No exchange rate available in SX [%s] for %s -> %s, value is zero.");
+                REPORT_ERROR(creation_data->creation_errors, err,
+                             xaccSchedXactionGetName(creation_data->sx),
+                             gnc_commodity_get_mnemonic(split_cmdty),
+                             gnc_commodity_get_mnemonic(first_cmdty));
                 final = gnc_numeric_zero();
             }
 
